//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "MainForm.h"
#include "VCLUtils.hpp"
#include "RxIni.hpp"
#include "pushkeys.h"
#include "win32x.h"
#include "findwinregexp.h"
//---------------------------------------------------------------------------
#pragma link "SpeedBar"
#pragma link "Placemnt"
#pragma resource "*.dfm"
TToolbarMainForm *ToolbarMainForm;

// defined in Toolbar.cpp
extern AnsiString AppTitle;
extern TRxIniFile *ini;

class TToolbarButton {
	public:
		TSpeedButton *b;
		AnsiString InsertText;	// text to type
		AnsiString Destination;	// destination window pattern to match (reg exp)
		TToolbarButton()	{	b = new TSpeedButton(ToolbarMainForm);	}
		~TToolbarButton()	{	delete b;				}
};
/*
class TToolbarButton : public TSpeedButton {
	public:
		AnsiString InsertText;	// text to type
		TToolbarButton() : TSpeedButton(ToolbarMainForm), InsertText("") {}
};
*/

TToolbarButton *buttons;	// an array of buttons (zero indexed)
int ButtonCount = 0;
bool CutAndPaste = false;	// add cut-and-paste keys to all macros
int ButtonWidth = 0;
int ButtonHeight = 0;
//---------------------------------------------------------------------------
int TextWidth(const AnsiString s, TFont *f)
/*
	Calculate the width of the string s displayed in the font f.
*/
{
	TCanvas *c = ToolbarMainForm->Canvas;
	c->Font = f;
	return c->TextWidth(s);
}
//---------------------------------------------------------------------------
int TextHeight(const AnsiString s, TFont *f)
/*
	Calculate the height of the string s displayed in the font f.
*/
{
	TCanvas *c = ToolbarMainForm->Canvas;
	c->Font = f;
	return c->TextHeight(s);
}
//---------------------------------------------------------------------------
__fastcall TToolbarMainForm::TToolbarMainForm(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TToolbarMainForm::FormCreate(TObject *Sender)
{
	// default settings
	FormPlacement1->IniFileName = ini->FileName;
	Caption = AppTitle;
	Application->Title = AppTitle;
	ShowHint = ini->ReadBool("Toolbar","ShowHints",false);
	CutAndPaste = ini->ReadBool("Toolbar","CutAndPaste",false);
	AnsiString Destination = ini->ReadString("Toolbar","Destination","");
	PushKeySleep = ini->ReadInteger("Toolbar","Sleep",0);	// v1.2b

	// stay on top? (v1.0c)
	bool onTop = ini->ReadBool("Toolbar","StayOnTop",true);
	if (onTop)	FormStyle = fsStayOnTop;
	else        FormStyle = fsNormal;

	// button on taskbar?  default = no if onTop, else yes (v1.0c)
	if (!ini->ReadBool("Toolbar","TaskbarButton",!onTop))	// if not button
		ShowWindowAsync(Application->Handle,SW_HIDE);		// hide from taskbar

	// button size
	ButtonWidth = ini->ReadInteger("Toolbar","ButtonSize",ButtonWidth);
	ButtonHeight= ini->ReadInteger("Toolbar","ButtonSize",ButtonHeight);

	// get button count from sections
	ButtonCount=1;
	AnsiString section = "Button1";
	while (!ini->ReadString(section,"InsertText","").IsEmpty()) {
		section = AnsiString("Button") + IntToStr(++ButtonCount);	// 1-indexed in ini-file
	}
	ButtonCount--;
	if (ButtonCount==0) {
		MsgBox(AppTitle,"Error: ButtonCount property not found in "
			"configuration file.",MB_OK);
		Close();
	}
	buttons = new TToolbarButton[ButtonCount];
	for (int i=0; i<ButtonCount; i++) {
		section = AnsiString("Button") + IntToStr(i+1);	// 1-indexed in ini-file
		buttons[i].b->Caption = ini->ReadString(section,"Caption","");
		buttons[i].b->Font    = ini->ReadFont(section,"Font",buttons[i].b->Font);
		buttons[i].b->Hint    = ini->ReadString(section,"Hint","");
		buttons[i].InsertText = ini->ReadString(section,"InsertText","");
		buttons[i].Destination= ini->ReadString(section,"Destination",
												Destination /*default*/);
		buttons[i].b->Tag     = i;
		buttons[i].b->OnClick = ButtonClick;
		InsertControl(buttons[i].b);
		ButtonWidth = max(ButtonWidth, TextWidth(buttons[i].b->Caption,
												 buttons[i].b->Font) + 4);
		ButtonHeight= max(ButtonHeight,TextHeight(buttons[i].b->Caption,
												  buttons[i].b->Font) + 4);
	}

	// resize buttons
	for (int i=0; i<ButtonCount; i++) {
		buttons[i].b->Width   = ButtonWidth;
		buttons[i].b->Height  = ButtonHeight;
	}
}
//---------------------------------------------------------------------------
void __fastcall TToolbarMainForm::FormClose(TObject *Sender,
	TCloseAction &Action)
{
	delete [] buttons;
}
//---------------------------------------------------------------------------
void __fastcall TToolbarMainForm::FormResize(TObject *Sender)
{
	int cols = max(1, Width / ButtonWidth);
	int col = 0, row = 0;
	for (int i=0; i<ButtonCount; i++) {
		buttons[i].b->Left = col * ButtonWidth;
		buttons[i].b->Top = row * ButtonHeight;
		col = (col+1) % cols;
		row += (col==0);
	}
}
//---------------------------------------------------------------------------
void __fastcall TToolbarMainForm::ButtonClick(TObject *Sender)
{
	// which button was pressed?
	int index = ((TSpeedButton *)Sender)->Tag;

	// send to which window?
	AnsiString Destination = buttons[index].Destination;

	// build list of keystrokes to send
	AnsiString keys = "";

	// first activate desired window (default to previously active)
	if (Destination.IsEmpty())
		keys += "%{TAB}";	// switch to previous window
	else {
		HWND dest = FindWindowRegexp("",Destination);
		if (dest == NULL || !IsWindow(dest))
			return;			// don't send keys if can't find window
		SetForegroundWindow(dest);
	}
	// if cut-and-pasting then be sure to empty clipboard first
	if (CutAndPaste)	keys += "{EMPTYCLIP}^x";	// empty and cut
	keys += buttons[index].InsertText;	// type
	if (CutAndPaste)	keys += "^v";	// paste
	// ok, let's do it!
	PushKeys(keys.c_str());
}
//---------------------------------------------------------------------------

